# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 
"""
$Id: ForumFolder.py,v 1.40.2.2 2004/05/17 03:13:41 fafhrd91 Exp $
"""

import traceback
from Acquisition import aq_base, aq_parent
from AccessControl import ClassSecurityInfo
from Products.CMFCore.interfaces.Dynamic import DynamicType
from Products.CMFCore.Skinnable import SkinnableObjectManager
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.PloneFolder import OrderedContainer
from Products.Archetypes.public import *
try:
    from Products.Archetypes.config import UUID_ATTR
except:
    UUID_ATTR = '_uid'

try:
    superGetAttr = OrderedBaseFolder.__getattr__
except:
    try:
        superGetAttr = OrderedBaseFolder.inheritedAttribute('__getattr__')
    except:
        superGetAttr = None

from config import predefined_skins
from uihelpers import UIHelpers
from CacheHooks import CacheHooks
from UserPreferences import UserPreferences
from ForumExtensibleSchema import \
     ForumSchemaHolder, ForumExtensibleSchema, clear_cache

_marker = []

class ForumFolder(ForumExtensibleSchema, OrderedBaseFolder, UserPreferences,
                  CacheHooks, ForumSchemaHolder, SkinnableObjectManager, UIHelpers,
                  OrderedContainer):
    """ """
    meta_type = 'ForumFolder'
    archetype_name = 'Forum Folder'

    __implements__ = (OrderedBaseFolder.__implements__ +
                      OrderedContainer.__implements__ + (DynamicType,))

    schema = BaseSchema + Schema((
        StringField('title', required=1, default='Discussion Forums',
                    widget=StringWidget(label_msgid='label_title',
                                        description_msgid='help_title',
                                        i18n_domain='plone')),

        ImageField('folder_icon', sizes={},
                   widget=ImageWidget(label='Forum folder icon',
                                      label_msgid='label_forum_folder_icon',
                                      description_msgid='help_forum_folder_icon',
                                      i18n_domain='CMFBoard')),
        StringField('predefined_skin', default='',
                    vocabulary = predefined_skins,
                    mutator = 'set_predefined_skin',
                    widget=SelectionWidget(label='Predefined skin',
                                           label_msgid='label_predefined_skin',
                                           description_msgid='help_predefined_skin',
                                           i18n_domain='plone')),

        BooleanField('use_custom_skin', default=0,
                     widget=BooleanWidget(label='Use custom skin',
                                          label_msgid='label_use_custom_skin',
                                          description_msgid='help_use_custom_skin',
                                          i18n_domain='plone')),

        StringField('custom_skin', default='',
                    mutator = 'set_custom_skin',
                    widget=StringWidget(label='Custom skin',
                                        label_msgid='label_custom_skin',
                                        description_msgid='help_custom_skin',
                                        i18n_domain='plone')),

        StringField('description', default='',
                    widget=TextAreaWidget(label_msgid='label_description',
                                          description_msgid='help_description',
                                          i18n_domain='plone')),
        ))


    allowed_content_types = ['ForumNB', 'ForumFolder']
    content_icon = 'fcFolder_icon.gif'

    _properties = (
        {'id': 'right_slots', 'type': 'lines', 'mode':'rwd'},
        )

    actions = (
            {'id': 'view',
             'name': 'View',
             'action': 'string:$object_url/forum_folder_view',
             'permissions': (View,)
             },
            )

    security = ClassSecurityInfo()

    right_slots = ()

    title = 'Discussion Forums'

    path1 = ''
    path2 = ''
    _initialized = 0

    security.declareProtected(View, 'Title')
    def Title(self):
        """ """
        return self.getField('title').get(self)

    def initializeArchetype(self, **kwargs):
        if kwargs.has_key('uid'):
            setattr(self, UUID_ATTR, kwargs['uid'])
        
        OrderedBaseFolder.initializeArchetype(self, **kwargs)
        try:
            if not self._initialized:
                self.forum_folder_default_init()
        except:
            pass
        
        self._initialized = 1

    def manage_afterDelete(self, item, container):
        OrderedBaseFolder.manage_afterDelete(self, item, container)

        self._copy = 0

    def _notifyOfCopyTo(self, container=None, op=1):
        OrderedBaseFolder._notifyOfCopyTo(self, container, op)

        self._copy = 1

        for ob in self.objectValues():
            ob._notifyOfCopyTo(container, op)

    def set_custom_skin(self, value, **kwargs):
        if self.getField('use_custom_skin').get(self):
            self.path1 = 'custom'
            self.path2 = value
        self.getField('custom_skin').set(self, value, **kwargs)

    def set_predefined_skin(self, value, **kwargs):
        self.path1 = 'cmfboard'
        self.path2 = value
        self.getField('predefined_skin').set(self, value, **kwargs)

        clear_cache(self.getPhysicalPath())

    security.declareProtected(View, 'manage_simpleSendMessage')
    def manage_simpleSendMessage(self, member_id, title, text):
        """simple_send method from a MailHost Object"""
        mtool = getToolByName(self, 'portal_membership')

        if mtool.isAnonymousUser():
            raise Exception, 'Only members can send emails.'

        member = mtool.getMemberById(member_id)
        if not member:
            raise Exception, "Can't find member."
        mto = member.getProperty('email', '')
        if not mto:
            raise Exception, "Member have not email address."

        sender = mtool.getAuthenticatedMember()
        mfrom = sender.getProperty('email', '')
        if not mfrom:
            raise Exception, "You have not email address."

        sfrom = sender.getProperty('fullname', '')
        if not sfrom:
            sfrom = str(sender)

        self.MailHost.simple_send(mto, '%s <%s>'%(sfrom, mfrom), title, text)


    #############################################
    ##  Skinnable API
    #############################################
    __of__ = SkinnableObjectManager.__of__
    _checkId = SkinnableObjectManager._checkId

    def getSkinsFolderName(self):
        return 'portal_skins'

    def __getattr__(self, name):
        if not name.startswith('_') and not name.startswith('aq_'):
            sd = self._v_skindata
            if sd is not None:
                request, ob, ignore, ob1 = sd
                if not ignore.has_key(name):
                    try:
                        if ob1 is not None:
                            subob = getattr(ob1, name, _marker)
                            if subob is not _marker:
                                return aq_base(subob)
                    except:
                        pass

                    subob = getattr(ob, name, _marker)
                    if subob is not _marker:
                        return aq_base(subob)
                    else:
                        ignore[name] = 1
        if superGetAttr is None:
            raise AttributeError, name
        return superGetAttr(self, name)

    security.declarePublic('changeSkin')
    def changeSkin(self, skinname):
        """ """
        self._v_skindata = None
        skinobj = self.getSkin(skinname)
        if skinobj is not None:
            ob1 = getattr(getattr(skinobj, self.path1, None), self.path2, None)
            self._v_skindata = (self.REQUEST, skinobj, {}, ob1)

    # hack around formcontroller bug
    security.declareProtected(View, 'validate')
    def validate(self, REQUEST=None, errors=None, data=None, metadata=None):
        """ """
        errors = OrderedBaseFolder.validate(self, REQUEST, errors, data, metadata)

        new_errors = {}
        for k, v in new_errors.items():
            if hasattr(v, '__getitem__'):
                new_errors[k] = v
        return new_errors


    #############################################
    ## CMFBoardCacheManager Hooks
    #############################################
    _cache_invalidates = {
        'message_new': ['forum_folder_view'],
        'message_deleted': ['forum_folder_view'],
        'message_changed': ['forum_folder_view'],
        'message_updated': ['forum_folder_view'],
        }

registerType(ForumFolder)


def modify_fti(fti):
    for a in fti['actions']:
        if a['id'] in ('references',):
            a['visible'] = 0
    return fti
